#include <string>
#include <iostream>
#include <fstream>
#include <allegro.h>
#include <math.h>

#define PI  3.14159265358979

using namespace std;

BITMAP *symetry(BITMAP *bmp)
{
    BITMAP *buf = create_bitmap(bmp->h, bmp->w);
    for(int y = 0 ; y < bmp->h ; y++)
        for(int x = 0 ; x < bmp->w ; x++)
            putpixel(buf, y, x, getpixel(bmp, x, y));
    return buf;
}

BITMAP *symetry_c(BITMAP *bmp)
{
    BITMAP *buf = create_bitmap(bmp->w, bmp->h);
    for(int y = 0 ; y < bmp->h ; y++)
        for(int x = 0 ; x < bmp->w ; x++)
            putpixel(buf, bmp->w - 1 - x, bmp->h - 1 - y, getpixel(bmp, x, y));
    return buf;
}

void save_image(BITMAP *bmp, const string &filename, int &X, int &Y)
{
    int mx = bmp->w - 1;
    int my = bmp->h - 1;
    int Mx = 0;
    int My = 0;
    for(int y = 0 ; y < bmp->h ; y++)
        for(int x = 0 ; x < bmp->w ; x++)
        {
            int alpha = geta(getpixel(bmp, x, y));
            if (alpha != 0)
            {
                if (y < my) my = y;
                if (y > My) My = y;
                if (x < mx) mx = x;
                if (x > Mx) Mx = x;
            }
        }
    X -= mx;
    Y -= my;
    int w = Mx - mx + 1;
    int h = My - my + 1;
    BITMAP *tmp = create_bitmap(w, h);
    blit(bmp, tmp, mx, my, 0, 0, w, h);
    save_bitmap(filename.c_str(), tmp, NULL);
    destroy_bitmap(tmp);
}

void draw_alpha_sprite(BITMAP *target, BITMAP *sprite, int x, int y)
{
    for(int Y = 0 ; Y < sprite->h ; Y++)
        for(int X = 0 ; X < sprite->w ; X++)
        {
            int c = getpixel(sprite, X, Y);
            int alpha = geta(c);
            if (alpha != 0)
                putpixel(target, x + X, y + Y, c | makeacol(0,0,0,0xFF));
        }
}

void draw_alpha_sprite_v_flip(BITMAP *target, BITMAP *sprite, int x, int y)
{
    for(int Y = 0 ; Y < sprite->h ; Y++)
        for(int X = 0 ; X < sprite->w ; X++)
        {
            int c = getpixel(sprite, X, sprite->h - 1 - Y);
            int alpha = geta(c);
            if (alpha != 0)
                putpixel(target, x + X, y + Y, c | makeacol(0,0,0,0xFF));
        }
}

void draw_alpha_sprite_h_flip(BITMAP *target, BITMAP *sprite, int x, int y)
{
    for(int Y = 0 ; Y < sprite->h ; Y++)
        for(int X = 0 ; X < sprite->w ; X++)
        {
            int c = getpixel(sprite, sprite->w - 1 - X, Y);
            int alpha = geta(c);
            if (alpha != 0)
                putpixel(target, x + X, y + Y, c | makeacol(0,0,0,0xFF));
        }
}

int main(int argc, char **argv)
{
    allegro_init();

    install_mouse();
    install_keyboard();

    set_color_depth(32);
    
    set_gfx_mode( GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0 );
    
    fstream file("cursors.gaf.txt", fstream::out);

    BITMAP *square = load_bitmap("images/carre.tga",NULL);
    BITMAP *circle = load_bitmap("images/circle.tga",NULL);
    BITMAP *blue = load_bitmap("images/blue.tga",NULL);
    BITMAP *red = load_bitmap("images/red.tga",NULL);
    BITMAP *yellow = load_bitmap("images/yellow.tga",NULL);
    BITMAP *ball6 = load_bitmap("images/boule6.tga",NULL);
    BITMAP *ball4 = load_bitmap("images/boule4.tga",NULL);

    BITMAP *bigblue = load_bitmap("images/bigblue.tga",NULL);
    BITMAP *broken0 = load_bitmap("images/broken0.tga",NULL);
    BITMAP *broken1 = load_bitmap("images/broken1.tga",NULL);

    BITMAP *bomb_y = load_bitmap("images/bombes_jaunes.tga",NULL);
    BITMAP *bomb_r = load_bitmap("images/bombes_rouges.tga",NULL);

    BITMAP *hourglass = load_bitmap("images/hourglass.tga",NULL);
    BITMAP *hourglass_rev = symetry_c(hourglass);

    BITMAP *attack_y0 = load_bitmap("images/attack_y0.tga",NULL);
    BITMAP *attack_r0 = load_bitmap("images/attack_r0.tga",NULL);
    BITMAP *attack_y1 = load_bitmap("images/attack_y1.tga",NULL);
    BITMAP *attack_r1 = load_bitmap("images/attack_r1.tga",NULL);
    BITMAP *attack_y2 = load_bitmap("images/attack_y2.tga",NULL);
    BITMAP *attack_r2 = load_bitmap("images/attack_r2.tga",NULL);
    BITMAP *attack_y3 = load_bitmap("images/attack_y3.tga",NULL);
    BITMAP *attack_r3 = load_bitmap("images/attack_r3.tga",NULL);

    BITMAP *move_arrow = load_bitmap("images/move_arrow.tga",NULL);
    BITMAP *move_arrow2 = symetry(move_arrow);

    BITMAP *buf = create_bitmap(128, 128);
    
    file << "[gadget0]" << endl;
    file << "{" << endl;
    file << "    filename=cursors.gaf;" << endl;
    file << "    entries=22;" << endl;
    file << "}" << endl;

    file << "[gadget1]" << endl;
    file << "{" << endl;
    file << "    frames=8;" << endl;
    file << "    name=cursormove;" << endl;
    for(int i = 0 ; i < 8 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);
        double amp = square->h / 8.0;
        double phase = PI;
        draw_alpha_sprite(buf, move_arrow, buf->w - move_arrow->w >> 1, buf->h / 2 - move_arrow->h - square->h / 2 + 4 - (int)((cos(i * PI / 4.0 + phase) + 1.0) * amp + 0.5));
        draw_alpha_sprite_v_flip(buf, move_arrow, buf->w - move_arrow->w >> 1, buf->h / 2 + square->h / 2 - 5 + (int)((cos(i * PI / 4.0 + phase) + 1.0) * amp + 0.5));

        draw_alpha_sprite(buf, move_arrow2, buf->w / 2 - move_arrow2->w - square->w / 2 + 4 - (int)((cos(i * PI / 4.0 + phase) + 1.0) * amp + 0.5), buf->h - move_arrow2->h >> 1);
        draw_alpha_sprite_h_flip(buf, move_arrow2, buf->w / 2 + square->w / 2 - 5 + (int)((cos(i * PI / 4.0 + phase) + 1.0) * amp + 0.5), buf->h - move_arrow2->h >> 1);
        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        int x = buf->w / 2, y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursormove%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursormove" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget2]" << endl;
    file << "{" << endl;
    file << "    frames=1;" << endl;
    file << "    name=cursorgrn;" << endl;
    file << "    [frame0]" << endl;
    file << "    {" << endl;
    file << "        XPos=0;" << endl;
    file << "        YPos=0;" << endl;
    file << "        filename=cursorgrn0.tga;" << endl;
    file << "    }" << endl;
    file << "}" << endl;

    file << "[gadget3]" << endl;
    file << "{" << endl;
    file << "    frames=2;" << endl;
    file << "    name=cursorselect;" << endl;
    file << "    [frame0]" << endl;
    file << "    {" << endl;
    file << "        XPos=10;" << endl;
    file << "        YPos=11;" << endl;
    file << "        filename=cursorselect0.tga;" << endl;
    file << "    }" << endl;
    file << "    [frame1]" << endl;
    file << "    {" << endl;
    file << "        XPos=10;" << endl;
    file << "        YPos=11;" << endl;
    file << "        filename=cursorselect1.tga;" << endl;
    file << "    }" << endl;
    file << "}" << endl;

    file << "[gadget4]" << endl;
    file << "{" << endl;
    file << "    frames=1;" << endl;
    file << "    name=cursorred;" << endl;
    file << "    [frame0]" << endl;
    file << "    {" << endl;
    file << "        XPos=0;" << endl;
    file << "        YPos=0;" << endl;
    file << "        filename=cursorred0.tga;" << endl;
    file << "    }" << endl;
    file << "}" << endl;

    file << "[gadget5]" << endl;
    file << "{" << endl;
    file << "    frames=16;" << endl;
    file << "    name=cursorload;" << endl;
    for(int i = 0 ; i < 16 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);
        int x = buf->w - square->w - blue->w >> 1;
        int y = buf->h - blue->h >> 1;
        double amp = 8.0;
        int steps = 11;
        int e = i < steps ? i : steps;
        draw_sprite(buf, blue, x - (int)(cos(e * PI / steps) * amp + 0.5), y - (int)(sin(e * PI / steps) * amp + 0.5));
        x = buf->w + square->w - 2 - blue->w >> 1;
        y = buf->h - blue->h >> 1;
        e = i >= 15 - steps ? i - (15 - steps) : 0;
        draw_sprite_h_flip(buf, blue, x + (int)(cos(e * PI / steps) * amp + 0.5), y - (int)(sin(e * PI / steps) * amp + 0.5));

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        x = buf->w / 2;
        y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorload%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorload" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget6]" << endl;
    file << "{" << endl;
    file << "    frames=18;" << endl;
    file << "    name=cursorrevive;" << endl;
    for(int i = 0 ; i < 18 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        int x = buf->w - blue->w >> 1;
        int y = buf->h - blue->h >> 1;
        
        int limit = blue->w * blue->h * i / 17;
        int ax = x, ay = y;
        int col = 0;
        for(int Y = 0 ; Y < blue->h && limit > 0 ; Y++)
            for(int X = 0 ; X < blue->w && limit > 0 ; X++, limit--)
            {
                putpixel(buf, x + X, y + Y, getpixel(blue, X, Y));
                ax = x + X;
                ay = y + Y;
                col = getpixel(blue, X, Y);
            }

        y = buf->h + square->h + blue->h >> 1;

        int ox = -1, oy = -1;

        limit = blue->w * blue->h * (17 - i) / 17;
        for(int Y = blue->h - 1 ; Y >= 0 && limit > 0 ; Y--)
            for(int X = blue->w - 1 ; X >= 0 && limit > 0 ; X--, limit--)
            {
                int pX = (X - (blue->w / 2)) * Y * 2 / (blue->h - 1) + (blue->w / 2);
                putpixel(buf, x + pX, y + Y / 3, getpixel(blue, X, Y));
                col = getpixel(blue, X, Y);
                ox = x + pX;
                oy = y + Y / 3;
            }

        if (ox != -1)
            line(buf, ox, oy, ax, ay, col);

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        x = buf->w / 2;
        y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorrevive%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorrevive" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget7]" << endl;
    file << "{" << endl;
    file << "    frames=16;" << endl;
    file << "    name=cursordefend;" << endl;
    for(int i = 0 ; i < 16 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        int a1 = 64 - i * 127 / 7;
        int a2 = 64 + i * 127 / 7;
        if (i >= 8)
        {
            a1 = 64 + (i-7) * 127 / 7;
            a2 = 64 - (i-7) * 127 / 7;
        }
        int r = 17;
        int c0 = makeacol(0, 0, 0, 255);
        int c1 = makeacol(180, 180, 210, 255);
        int c2 = makeacol(0, 0, 0, 255);

        if (i < 15)
        {
            arc(buf, buf->w / 2 - 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r+1, c0);
            arc(buf, buf->w / 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r+1, c0);
            arc(buf, buf->w / 2 - 1, buf->h / 2 - 1, itofix(a1), itofix(a2), r+1, c0);

            arc(buf, buf->w / 2 - 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r-1, c2);
            arc(buf, buf->w / 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r-1, c2);
            arc(buf, buf->w / 2 - 1, buf->h / 2 - 1, itofix(a1), itofix(a2), r-1, c2);

            arc(buf, buf->w / 2 - 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r, c1);
            arc(buf, buf->w / 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r, c1);
            arc(buf, buf->w / 2 - 1, buf->h / 2 - 1, itofix(a1), itofix(a2), r, c1);
        }

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        int x = buf->w / 2;
        int y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursordefend%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursordefend" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget8]" << endl;
    file << "{" << endl;
    file << "    frames=14;" << endl;
    file << "    name=cursorpatrol;" << endl;
    for(int i = 0 ; i < 14 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        double amp = 16.0;

        if (i < 4 || i > 10)
            draw_sprite(buf, ball6, buf->w / 2 - ball6->w / 2 - 1 - (int)(amp + 0.5), buf->h / 2 - ball6->h / 2 - 1);
        else
            draw_sprite(buf, ball6, buf->w / 2 - ball6->w / 2 - 1 + (int)(amp + 0.5), buf->h / 2 - ball6->h / 2 - 1);

        for(int e = 0 ; e < 3 ; e++)
        {
            double phase = (i + e - 3) * 2.0 * PI / 8.0 - PI;
            double phase0 = (i - 3) * 2.0 * PI / 8.0 - PI;
            if (i < 7)
            {
                if (phase >= 0 && phase0 < 0)  continue;
                if (phase <= -PI)  continue;
            }
            else
            {
                phase = (i + e - 6) * 2.0 * PI / 8.0 - PI;
                phase0 = (i - 6) * 2.0 * PI / 8.0 - PI;
                if (phase >= PI && phase0 <= PI)  continue;
                if (phase <= 0)  continue;
            }
            int x = (buf->w / 2) + (int)(cos(phase) * amp + 0.5);
            int y = (buf->h / 2) + (int)(sin(phase) * amp + 0.5);
            draw_sprite(buf, ball4, x - ball4->w / 2 - 1, y - ball4->h / 2 - 1);
        }

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        int x = buf->w / 2;
        int y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorpatrol%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorpatrol" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget9]" << endl;
    file << "{" << endl;
    file << "    frames=8;" << endl;
    file << "    name=cursorprotect;" << endl;
    for(int i = 0 ; i < 8 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        int a1 = 0 - i * 255 / 8;
        int a2 = a1 + 64;
        int r = 17;
        int c0 = makeacol(255, 155, 0, 255);
        int c1 = makeacol(255, 50, 50, 255);
        int c2 = makeacol(255, 155, 0, 255);

        if (i < 15)
        {
            arc(buf, buf->w / 2 - 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r+1, c0);
            arc(buf, buf->w / 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r+1, c0);
            arc(buf, buf->w / 2 - 1, buf->h / 2 - 1, itofix(a1), itofix(a2), r+1, c0);

            arc(buf, buf->w / 2 - 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r-1, c2);
            arc(buf, buf->w / 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r-1, c2);
            arc(buf, buf->w / 2 - 1, buf->h / 2 - 1, itofix(a1), itofix(a2), r-1, c2);

            arc(buf, buf->w / 2 - 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r, c1);
            arc(buf, buf->w / 2, buf->h / 2 - 1, itofix(a1), itofix(a2), r, c1);
            arc(buf, buf->w / 2 - 1, buf->h / 2 - 1, itofix(a1), itofix(a2), r, c1);
        }

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        int x = buf->w / 2;
        int y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorprotect%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorprotect" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget10]" << endl;
    file << "{" << endl;
    file << "    frames=12;" << endl;
    file << "    name=cursorrepair;" << endl;
    for(int i = 0 ; i < 12 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        if (i < 10)
        {
            draw_alpha_sprite(buf, broken0, (buf->w - broken0->w) / 2 - 10 * (9 - i) / 9, buf->h - broken0->h >> 1);
            draw_alpha_sprite(buf, broken1, (buf->w - broken1->w) / 2 + 10 * (9 - i) / 9, buf->h - broken0->h >> 1);
        }
        else
            draw_alpha_sprite(buf, bigblue, buf->w - bigblue->w >> 1, buf->h - bigblue->h >> 1);

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        int x = buf->w / 2;
        int y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorrepair%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorrepair" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget11]" << endl;
    file << "{" << endl;
    file << "    frames=10;" << endl;
    file << "    name=cursorattack;" << endl;
    for(int i = 0 ; i < 10 ; i++)
    {
        clear(buf);
        draw_sprite(buf, circle, buf->w - circle->w >> 1, buf->h - circle->h >> 1);

        int x = (buf->w - attack_y0->w >> 1);
        int y = (buf->h - attack_y0->h >> 1);
        int e = 10 - (int)(cos(i * 2.0 * PI / 9.0 + PI) * 2.0 + 0.5);
        if (!(i&1))
        {
            draw_alpha_sprite(buf, attack_y0, x - 1 - 10 * e / 9, y - 1 - 10 * e / 9);
            draw_alpha_sprite(buf, attack_y1, x + 10 * e / 9, y - 1 - 10 * e / 9);
            draw_alpha_sprite(buf, attack_y2, x - 1 - 10 * e / 9, y + 10 * e / 9);
            draw_alpha_sprite(buf, attack_y3, x + 10 * e / 9, y + 10 * e / 9);
        }
        else
        {
            draw_alpha_sprite(buf, attack_r0, x - 1 - 10 * e / 9, y - 1 - 10 * e / 9);
            draw_alpha_sprite(buf, attack_r1, x + 10 * e / 9, y - 1 - 10 * e / 9);
            draw_alpha_sprite(buf, attack_r2, x - 1 - 10 * e / 9, y + 10 * e / 9);
            draw_alpha_sprite(buf, attack_r3, x + 10 * e / 9, y + 10 * e / 9);
        }

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        x = buf->w / 2;
        y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorattack%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorattack" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget12]" << endl;
    file << "{" << endl;
    file << "    frames=1;" << endl;
    file << "    name=cursornormal;" << endl;
    file << "    [frame0]" << endl;
    file << "    {" << endl;
    file << "        XPos=0;" << endl;
    file << "        YPos=0;" << endl;
    file << "        filename=cursornormal0.tga;" << endl;
    file << "    }" << endl;
    file << "}" << endl;
    
    file << "[gadget13]" << endl;
    file << "{" << endl;
    file << "    frames=24;" << endl;
    file << "    name=cursorpickup;" << endl;
    for(int i = 0 ; i < 24 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        int x = buf->w - move_arrow->w >> 1;
        int y = (buf->h - move_arrow->h >> 1) - 3 - (int)(cos(i * 2.0 * PI / 23.0) * 6.0 + 0.5);
        draw_alpha_sprite_v_flip(buf, move_arrow, x, y);

        x = buf->w - blue->w >> 1;
        if ( i < 14)
            y = (buf->h >> 1) + 8;
        else
            y = (buf->h >> 1) + 2 - (int)(cos(i * 2.0 * PI / 23.0) * 6.0 + 0.5);
        draw_alpha_sprite(buf, blue, x, y);

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        x = buf->w / 2;
        y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorpickup%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorpickup" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget14]" << endl;
    file << "{" << endl;
    file << "    frames=16;" << endl;
    file << "    name=cursorairstrike;" << endl;
    for(int i = 0 ; i < 16 ; i++)
    {
        clear(buf);
        draw_sprite(buf, circle, buf->w - circle->w >> 1, buf->h - circle->h >> 1);

        int x = buf->w - bomb_y->w >> 1;
        int y = (buf->h - bomb_y->h >> 1) + i - 10;

        if (i&1)
            draw_alpha_sprite(buf, bomb_y, x, y);
        else
            draw_alpha_sprite(buf, bomb_r, x, y);

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        x = buf->w / 2;
        y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorairstrike%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorairstrike" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget15]" << endl;
    file << "{" << endl;
    file << "    frames=" << blue->h * 6 << ";" << endl;
    file << "    name=cursorteleport;" << endl;
    for(int i = 0 ; i < blue->h * 6 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        int e = i < blue->h * 3 ? i : blue->h * 6 - i;

        int x = (buf->w - square->w - blue->w >> 1) - 5;
        int y = (buf->h - blue->h >> 1);
        int p = e - blue->h;
        if (p < 0)  p = 0;
        if (p < blue->h)
            rectfill(buf, x, y + p, x + blue->w - 1, y + blue->h - 1, makeacol(180, 180, 210, 255));
        p = e;
        blit(blue, buf, 0, p, x, y + p, blue->w, blue->h - p);

        x = (buf->w + square->w - blue->w >> 1) + 5;
        y = (buf->h - blue->h >> 1);
        p = e - blue->h;
        p = blue->h - p;
        if (p < 0)  p = 0;
        if (p < blue->h)
            rectfill(buf, x, y + p, x + blue->w - 1, y + blue->h - 1, makeacol(180, 180, 210, 255));
        p = blue->h * 3 - e;
        blit(blue, buf, 0, p, x, y + p, blue->w, blue->h - p);

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        x = buf->w / 2;
        y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorteleport%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorteleport" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget16]" << endl;
    file << "{" << endl;
    file << "    frames=11;" << endl;
    file << "    name=cursorreclamate;" << endl;
    for(int i = 0 ; i < 11 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        if (i < 8)
        {
            int x = buf->w >> 1;
            int y = buf->h >> 1;
            for(int Y = 0 ; Y < blue->h ; Y++)
                for(int X = 0 ; X < blue->w ; X++)
                {
                    int c = getpixel(blue, X, Y);
                    if (geta(c) == 0)   continue;
                    double dx = X - (blue->w * 0.5 - 0.25);
                    double dy = Y - (blue->h * 0.5 - 0.25);
                    double l = sqrt(dx * dx + dy * dy);
                    if (l == 0.0)  l = 1.0;
                    double p = 1.0 + 12.0 * (7.0 - i) / 7.0 / l;
                    putpixel(buf, x + dx * p, y + dy * p, c);
                }
        }
        else
            draw_sprite(buf, blue, buf->w - blue->w >> 1, buf->h - blue->h >> 1);

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        int x = buf->w / 2;
        int y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorreclamate%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorreclamate" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;
    
    file << "[gadget17]" << endl;
    file << "{" << endl;
    file << "    frames=2;" << endl;
    file << "    name=cursorfindsite;" << endl;
    file << "    [frame0]" << endl;
    file << "    {" << endl;
    file << "        XPos=10;" << endl;
    file << "        YPos=11;" << endl;
    file << "        filename=cursorselect0.tga;" << endl;
    file << "    }" << endl;
    file << "    [frame1]" << endl;
    file << "    {" << endl;
    file << "        XPos=10;" << endl;
    file << "        YPos=11;" << endl;
    file << "        filename=cursorselect1.tga;" << endl;
    file << "    }" << endl;
    file << "}" << endl;

    file << "[gadget18]" << endl;
    file << "{" << endl;
    file << "    frames=13;" << endl;
    file << "    name=cursorcapture;" << endl;
    for(int i = 0 ; i < 13 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);

        if (i < 9)
        {
            int x = buf->w - yellow->w >> 1;
            int y = buf->h - yellow->h >> 1;
            if (!(i&1))
            {
                draw_sprite(buf, yellow, x - 3 * (9 - i) / 2, y);
                draw_sprite(buf, yellow, x + 3 * (9 - i) / 2, y);
            }
            else
            {
                draw_sprite(buf, red, x - 3 * (9 - i) / 2, y);
                draw_sprite(buf, red, x + 3 * (9 - i) / 2, y);
            }
        }
        else
            draw_sprite(buf, blue, buf->w - blue->w >> 1, buf->h - blue->h >> 1);

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        int x = buf->w / 2;
        int y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorcapture%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorcapture" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget19]" << endl;
    file << "{" << endl;
    file << "    frames=16;" << endl;
    file << "    name=cursorunload;" << endl;
    for(int i = 0 ; i < 16 ; i++)
    {
        clear(buf);
        draw_sprite(buf, square, buf->w - square->w >> 1, buf->h - square->h >> 1);
        int x = buf->w - square->w - blue->w >> 1;
        int y = buf->h - blue->h >> 1;
        double amp = 8.0;
        int steps = 11;
        int f = 15 - i;
        int e = f < steps ? f : steps;
        draw_sprite(buf, blue, x - (int)(cos(e * PI / steps) * amp + 0.5), y - (int)(sin(e * PI / steps) * amp + 0.5));
        x = buf->w + square->w - 2 - blue->w >> 1;
        y = buf->h - blue->h >> 1;
        e = f >= 15 - steps ? f - (15 - steps) : 0;
        draw_sprite_h_flip(buf, blue, x + (int)(cos(e * PI / steps) * amp + 0.5), y - (int)(sin(e * PI / steps) * amp + 0.5));

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        x = buf->w / 2;
        y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorunload%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorunload" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget20]" << endl;
    file << "{" << endl;
    file << "    frames=8;" << endl;
    file << "    name=cursorhourglass;" << endl;
    for(int i = 0 ; i < 8 ; i++)
    {
        clear(buf);
        if (i < 5)
        {
            int p = i + 3;
            blit(hourglass_rev, buf, 0, 0, buf->w - hourglass->w >> 1, buf->h - hourglass->h >> 1, hourglass->w, p);
            blit(hourglass, buf, 0, p, buf->w - hourglass->w >> 1, (buf->h - hourglass->h >> 1) + p, hourglass->w, hourglass->h - p);
            p = hourglass->h - i - 3;
            blit(hourglass_rev, buf, 0, p, buf->w - hourglass->w >> 1, (buf->h - hourglass->h >> 1) + p, hourglass->w, hourglass->h - p);
        }
        else
            rotate_sprite(buf, hourglass_rev, buf->w - hourglass->w >> 1, buf->h - hourglass->h >> 1, itofix(128 * (i-4) / 4));

        clear(screen);
        blit(buf, screen, 0, 0, 0, 0, buf->w, buf->h);
        
        int x = buf->w / 2;
        int y = buf->h / 2;
        char tmp[100];
        sprintf(tmp, "cursorhourglass%d.tga", i);
        save_image(buf, tmp, x, y);
        file << "    [frame" << i << "]" << endl;
        file << "    {" << endl;
        file << "        XPos=" << x << ";" << endl;
        file << "        YPos=" << y << ";" << endl;
        file << "        filename=cursorhourglass" << i << ".tga;" << endl;
        file << "    }" << endl;
    }
    file << "}" << endl;

    file << "[gadget21]" << endl;
    file << "{" << endl;
    file << "    frames=2;" << endl;
    file << "    name=cursortoofar;" << endl;
    file << "    [frame0]" << endl;
    file << "    {" << endl;
    file << "        XPos=14;" << endl;
    file << "        YPos=14;" << endl;
    file << "        filename=cursortoofar0.tga;" << endl;
    file << "    }" << endl;
    file << "    [frame1]" << endl;
    file << "    {" << endl;
    file << "        XPos=14;" << endl;
    file << "        YPos=14;" << endl;
    file << "        filename=cursortoofar1.tga;" << endl;
    file << "    }" << endl;
    file << "}" << endl;
    file << "[gadget22]" << endl;
    file << "{" << endl;
    file << "    frames=1;" << endl;
    file << "    name=pathicon;" << endl;
    file << "    [frame0]" << endl;
    file << "    {" << endl;
    file << "        XPos=5;" << endl;
    file << "        YPos=5;" << endl;
    file << "        filename=pathicon0.tga;" << endl;
    file << "    }" << endl;
    file << "}" << endl;
    file.close();

    allegro_exit();
    return 0;
}
END_OF_MAIN();
